@@ -71,6 +71,8 @@ def get_data_multiplication_queue(model_description_multiply):
 
 class Multiply(object):
 
+    """Copy the data n times."""
+
     def __init__(self, nr=1):
         self.nr = nr
 
@@ -93,6 +95,8 @@ class Multiply(object):
 
 class Rotate(object):
 
+    """Add rotational variants of the recording."""
+
     def __init__(self, minimum=-30.0, maximum=30.0, num=5.0):
         self.min = minimum
         self.max = maximum
