@@ -12,10 +12,14 @@ import yaml
 import natsort
 import time
 import datetime
+import subprocess
 # mine
 import preprocess_dataset
+import features
 import create_pfiles
 import create_model
+import get_error
+from HandwrittenData import HandwrittenData
 
 
 def print_status(total, current, start_time=None):
@@ -321,3 +325,81 @@ def default_model():
     else:
         latest_model = get_latest_folder(models_dir)
     return latest_model
+
+
+def evaluate_model(recording, model_folder):
+    """Evaluate model for a single recording."""
+
+    folders = []
+    folder = model_folder
+    while os.path.isdir(folder):
+        folders.append(folder)
+        # Get info.yml
+        with open(os.path.join(folder, "info.yml")) as ymlfile:
+            content = yaml.load(ymlfile)
+        folder = os.path.join(get_project_root(), content['data-source'])
+    raw_source_file = folder
+    dt = os.path.getmtime(raw_source_file)
+    folders = folders[::-1]  # Reverse order to get the most "basic one first"
+
+    for target_folder in folders:
+        # The source is later than the target. That means we need to
+        # refresh the target
+        if "preprocessed" in target_folder:
+            logging.info("Start applying preprocessing methods")
+            t = target_folder
+            _, _, preprocessing_queue = preprocess_dataset.get_parameters(t)
+            handwriting = HandwrittenData(recording)
+            handwriting.preprocessing(preprocessing_queue)
+        elif "feature-files" in target_folder:
+            logging.info("Feature file was outdated. Update...")
+            infofile_path = os.path.join(target_folder, "info.yml")
+            with open(infofile_path, 'r') as ymlfile:
+                feature_description = yaml.load(ymlfile)
+            feature_str_list = feature_description['features']
+            feature_list = features.get_features(feature_str_list)
+            feature_count = sum(map(lambda n: n.get_dimension(),
+                                    feature_list))
+            x = handwriting.feature_extraction(feature_list)
+            # Create pfile
+            input_filename = "tmp.txt"
+            output_filename = "evaluate.pfile"
+            with open(input_filename, "w") as f:
+                for symbolnr, instance in enumerate([x, "0"]):
+                    feature_string, label = instance
+                    assert len(feature_string) == feature_count, \
+                        "Expected %i features, got %i features" % \
+                        (feature_count, len(feature_string))
+                    feature_string = " ".join(map(str, feature_string))
+                    line = "%i 0 %s %i" % (symbolnr, feature_string, label)
+                    print(line, file=f)
+            command = "pfile_create -i %s -f %i -l 1 -o %s" % \
+                      (input_filename, feature_count, output_filename)
+            logging.info(command)
+            os.system(command)
+        elif "model" in target_folder:
+            model_src = get_latest_model(target_folder, "model")
+            model_use = "tmp.json"
+            get_error.create_adjusted_model_for_percentages(model_src,
+                                                            model_use)
+            # Run evaluation
+            PROJECT_ROOT = get_project_root()
+            time_prefix = time.strftime("%Y-%m-%d-%H-%M")
+            test_file = output_filename
+            logging.info("Evaluate '%s' with '%s'...", model_src, test_file)
+            logfile = os.path.join(PROJECT_ROOT,
+                                   "logs/%s-error-evaluation.log" %
+                                   time_prefix)
+            with open(logfile, "w") as log, open(model_use, "r") as modl_src_p:
+                p = subprocess.Popen(['nntoolkit', 'run',
+                                      '--batch-size', '1',
+                                      '-f%0.4f', test_file],
+                                     stdin=modl_src_p,
+                                     stdout=log)
+                ret = p.wait()
+                if ret != 0:
+                    logging.error("nntoolkit finished with ret code %s", str(ret))
+                    sys.exit()
+            return logfile
+        else:
+            logging.info("'%s' not found", target_folder)
